@@ -147,71 +147,77 @@ gtd_provider_eds_on_client_connected (GObject      *source_object,
 {
   GtdProviderEdsPrivate *priv;
   GtdProviderEds *self;
+  GtdTaskListEds *list;
   ECalClient *client;
+  TaskData *data;
+  ESource *default_source;
   ESource *source;
-  GError *error = NULL;
+  ESource *parent;
+  GError *error;
 
   self = GTD_PROVIDER_EDS (user_data);
   priv = gtd_provider_eds_get_instance_private (self);
+  error = NULL;
   source = e_client_get_source (E_CLIENT (source_object));
   client = E_CAL_CLIENT (e_cal_client_connect_finish (result, &error));
 
-  if (!error)
-    {
-      GtdTaskListEds *list;
-      TaskData *data;
-      ESource *parent;
-
-      /* parent source's display name is list's origin */
-      parent = e_source_registry_ref_source (priv->source_registry, e_source_get_parent (source));
-
-      /* creates a new task list */
-      list = gtd_task_list_eds_new (GTD_PROVIDER (self), source);
-
-      /* it's not ready until we fetch the list of tasks from client */
-      gtd_object_set_ready (GTD_OBJECT (list), FALSE);
-
-      /* async data */
-      data = task_data_new (user_data, (gpointer) list);
-
-      /* asyncronously fetch the task list */
-      e_cal_client_get_object_list_as_comps (client,
-                                             "contains? \"any\" \"\"",
-                                             NULL,
-                                             gtd_provider_eds_fill_task_list,
-                                             data);
-
-      priv->task_lists = g_list_append (priv->task_lists, list);
-
-      g_object_set_data (G_OBJECT (source), "task-list", list);
-      g_hash_table_insert (priv->clients, source, client);
-
-      /* Emit LIST_ADDED signal */
-      g_signal_emit_by_name (self, "list-added", list);
-
-      g_object_unref (parent);
-
-      g_debug ("%s: %s (%s)",
-               G_STRFUNC,
-               _("Task list source successfully connected"),
-               e_source_get_display_name (source));
-    }
-  else
+  if (error)
     {
       g_debug ("%s: %s (%s): %s",
                G_STRFUNC,
-               _("Failed to connect to task list source"),
+               _("Failed to connect to task list"),
                e_source_get_uid (source),
                error->message);
 
       gtd_manager_emit_error_message (gtd_manager_get_default (),
-                                      _("Failed to connect to task list source"),
+                                      _("Failed to connect to task list"),
                                       error->message);
 
       g_error_free (error);
       return;
     }
 
+  /* parent source's display name is list's origin */
+  parent = e_source_registry_ref_source (priv->source_registry, e_source_get_parent (source));
+
+  /* creates a new task list */
+  list = gtd_task_list_eds_new (GTD_PROVIDER (self), source);
+
+  /* it's not ready until we fetch the list of tasks from client */
+  gtd_object_set_ready (GTD_OBJECT (list), FALSE);
+
+  /* async data */
+  data = task_data_new (user_data, (gpointer) list);
+
+  /* asyncronously fetch the task list */
+  e_cal_client_get_object_list_as_comps (client,
+                                         "contains? \"any\" \"\"",
+                                         NULL,
+                                         gtd_provider_eds_fill_task_list,
+                                         data);
+
+  priv->task_lists = g_list_append (priv->task_lists, list);
+
+  g_object_set_data (G_OBJECT (source), "task-list", list);
+  g_hash_table_insert (priv->clients, source, client);
+
+  /* Check if the current list is the default one */
+  default_source = e_source_registry_ref_default_task_list (priv->source_registry);
+
+  if (default_source == source)
+    g_object_notify (G_OBJECT (self), "default-task-list");
+
+  g_clear_object (&default_source);
+
+  /* Emit LIST_ADDED signal */
+  g_signal_emit_by_name (self, "list-added", list);
+
+  g_object_unref (parent);
+
+  g_debug ("%s: %s (%s)",
+           G_STRFUNC,
+           _("Task list source successfully connected"),
+           e_source_get_display_name (source));
 }
 
 typedef struct
@@ -402,6 +408,26 @@ gtd_provider_eds_credentials_required (ESourceRegistry          *registry,
     }
 }
 
+static void
+default_tasklist_changed_cb (ESourceRegistry *source_registry,
+                             GParamSpec      *pspec,
+                             GtdProviderEds  *self)
+{
+  GtdTaskList *list;
+  ESource *default_source;
+
+  default_source = e_source_registry_ref_default_task_list (source_registry);
+  list = g_object_get_data (G_OBJECT (default_source), "task-list");
+
+  /* The list might not be loaded yet */
+  if (!list)
+    goto out;
+
+  g_object_notify (G_OBJECT (self), "default-task-list");
+
+out:
+  g_clear_object (&default_source);
+}
 
 static void
 gtd_provider_eds_finalize (GObject *object)
@@ -409,6 +435,8 @@ gtd_provider_eds_finalize (GObject *object)
   GtdProviderEds *self = (GtdProviderEds *)object;
   GtdProviderEdsPrivate *priv = gtd_provider_eds_get_instance_private (self);
 
+  g_signal_handlers_disconnect_by_func (priv->source_registry, default_tasklist_changed_cb, self);
+
   g_clear_pointer (&priv->clients, g_hash_table_destroy);
   g_clear_object (&priv->credentials_prompter);
   g_clear_object (&priv->source_registry);
@@ -492,6 +520,11 @@ gtd_provider_eds_load_registry (GtdProviderEds  *provider)
                     G_CALLBACK (gtd_provider_eds_credentials_required),
                     provider);
 
+  g_signal_connect (priv->source_registry,
+                    "notify::default-task-list",
+                    G_CALLBACK (default_tasklist_changed_cb),
+                    provider);
+
   e_credentials_prompter_process_awaiting_credentials (priv->credentials_prompter);
 }
 
@@ -676,6 +709,8 @@ task_list_removal_finished (GtdProvider *provider,
 {
   gtd_object_set_ready (GTD_OBJECT (provider), TRUE);
 
+  g_object_notify (G_OBJECT (provider), "default-task-list");
+
   if (*error)
     {
       g_warning ("%s: %s: %s",
@@ -1039,8 +1074,26 @@ gtd_provider_eds_get_default_task_list (GtdProviderEds *provider)
 
   g_clear_object (&default_source);
 
-  if (gtd_task_list_get_provider (default_task_list) != GTD_PROVIDER (provider))
-    return NULL;
+  if (default_task_list &&
+      gtd_task_list_get_provider (default_task_list) != GTD_PROVIDER (provider))
+    {
+      return NULL;
+    }
 
   return default_task_list;
 }
+
+void
+gtd_provider_eds_set_default_task_list (GtdProviderEds *provider,
+                                        GtdTaskList    *list)
+{
+  GtdProviderEdsPrivate *priv;
+  ESource *source;
+
+  priv = gtd_provider_eds_get_instance_private (provider);
+  source = g_object_get_data (G_OBJECT (list), "task-list");
+
+  e_source_registry_set_default_task_list (priv->source_registry, source);
+
+  g_object_notify (G_OBJECT (provider), "default-task-list");
+}
