@@ -22,6 +22,7 @@
 #include "gtd-empty-list-widget.h"
 #include "gtd-task-list-view.h"
 #include "gtd-manager.h"
+#include "gtd-new-task-row.h"
 #include "gtd-notification.h"
 #include "gtd-provider.h"
 #include "gtd-task.h"
@@ -69,7 +70,7 @@ typedef struct
   GtkRevealer           *edit_revealer;
   GtkWidget             *empty_box;
   GtkListBox            *listbox;
-  GtdTaskRow            *new_task_row;
+  GtkListBoxRow         *new_task_row;
   GtkRevealer           *revealer;
   GtkImage              *done_image;
   GtkLabel              *done_label;
@@ -79,8 +80,9 @@ typedef struct
   /* internal */
   gboolean               can_toggle;
   gint                   complete_tasks;
-  gboolean               show_list_name;
-  gboolean               show_completed;
+  gboolean               show_completed : 1;
+  gboolean               show_due_date : 1;
+  gboolean               show_list_name : 1;
   gboolean               handle_subtasks : 1;
   GList                 *list;
   GtdTaskList           *task_list;
@@ -105,7 +107,9 @@ typedef struct
   GtdTaskListViewSortFunc sort_func;
   gpointer                sort_user_data;
 
-  GtkWidget             *active_row;
+  GtkWidget              *active_row;
+  GtkSizeGroup           *due_date_sizegroup;
+  GtkSizeGroup           *tasklist_name_sizegroup;
 } GtdTaskListViewPrivate;
 
 struct _GtdTaskListView
@@ -158,6 +162,7 @@ enum {
   PROP_HANDLE_SUBTASKS,
   PROP_SHOW_COMPLETED,
   PROP_SHOW_LIST_NAME,
+  PROP_SHOW_DUE_DATE,
   PROP_SHOW_NEW_TASK_ROW,
   LAST_PROP
 };
@@ -178,12 +183,24 @@ set_active_row (GtdTaskListView *self,
     return;
 
   if (priv->active_row)
-    gtd_task_row_set_active (GTD_TASK_ROW (priv->active_row), FALSE);
+    {
+      if (GTD_IS_TASK_ROW (priv->active_row))
+        gtd_task_row_set_active (GTD_TASK_ROW (priv->active_row), FALSE);
+      else
+        gtd_new_task_row_set_active (GTD_NEW_TASK_ROW (priv->active_row), FALSE);
+    }
 
   priv->active_row = row;
 
   if (row)
-    gtd_task_row_set_active (GTD_TASK_ROW (row), TRUE);
+    {
+      if (GTD_IS_TASK_ROW (row))
+        gtd_task_row_set_active (GTD_TASK_ROW (row), TRUE);
+      else
+        gtd_new_task_row_set_active (GTD_NEW_TASK_ROW (row), TRUE);
+
+      gtk_widget_grab_focus (row);
+    }
 }
 
 /*
@@ -330,34 +347,41 @@ undo_remove_task_action (GtdNotification *notification,
  * Default sorting functions
  */
 static gint
-compare_task_rows (GtdTaskRow *row1,
-                   GtdTaskRow *row2)
+compare_task_rows (GtkListBoxRow *row1,
+                   GtkListBoxRow *row2)
 {
-  if (gtd_task_row_get_new_task_mode (row1))
-    return 1;
-  else if (gtd_task_row_get_new_task_mode (row2))
-    return -1;
+  if (GTD_IS_NEW_TASK_ROW (row1))
+    {
+      return 1;
+    }
+  else if (GTD_IS_NEW_TASK_ROW (row2))
+    {
+      return -1;
+    }
   else
-    return gtd_task_compare (gtd_task_row_get_task (row1), gtd_task_row_get_task (row2));
+    {
+      return gtd_task_compare (gtd_task_row_get_task (GTD_TASK_ROW (row1)),
+                               gtd_task_row_get_task (GTD_TASK_ROW (row2)));
+    }
 }
 
 static gint
 compare_dnd_rows (GtkListBoxRow *row1,
                   GtkListBoxRow *row2)
 {
-  GtdTaskRow *row_above, *current_row;
+  GtkListBoxRow *row_above, *current_row;
   gboolean reverse;
 
   if (GTD_IS_DND_ROW (row1))
     {
       row_above = gtd_dnd_row_get_row_above (GTD_DND_ROW (row1));
-      current_row = GTD_TASK_ROW (row2);
+      current_row = row2;
       reverse = FALSE;
     }
   else
     {
       row_above = gtd_dnd_row_get_row_above (GTD_DND_ROW (row2));
-      current_row = GTD_TASK_ROW (row1);
+      current_row = row1;
       reverse = TRUE;
     }
 
@@ -379,15 +403,15 @@ gtd_task_list_view__listbox_sort_func (GtkListBoxRow *row1,
   if (GTD_IS_DND_ROW (row1) || GTD_IS_DND_ROW (row2))
     return compare_dnd_rows (row1, row2);
 
-  return compare_task_rows (GTD_TASK_ROW (row1), GTD_TASK_ROW (row2));
+  return compare_task_rows (row1, row2);
 }
 
 /*
  * Custom sorting functions
  */
 static void
-internal_header_func (GtdTaskRow      *row,
-                      GtdTaskRow      *before,
+internal_header_func (GtkListBoxRow   *row,
+                      GtkListBoxRow   *before,
                       GtdTaskListView *view)
 {
   GtdTask *row_task;
@@ -398,11 +422,11 @@ internal_header_func (GtdTaskRow      *row,
 
   row_task = before_task = NULL;
 
-  if (row)
-    row_task = gtd_task_row_get_task (row);
+  if (row && GTD_IS_TASK_ROW (row))
+    row_task = gtd_task_row_get_task (GTD_TASK_ROW (row));
 
-  if (before)
-    before_task = gtd_task_row_get_task (before);
+  if (before && GTD_IS_TASK_ROW (row))
+    before_task = gtd_task_row_get_task (GTD_TASK_ROW (before));
 
   view->priv->header_func (GTK_LIST_BOX_ROW (row),
                            row_task,
@@ -413,23 +437,23 @@ internal_header_func (GtdTaskRow      *row,
 
 static gint
 internal_compare_task_rows (GtdTaskListView *self,
-                            GtdTaskRow      *row1,
-                            GtdTaskRow      *row2)
+                            GtkListBoxRow   *row1,
+                            GtkListBoxRow   *row2)
 {  GtdTask *row1_task;
   GtdTask *row2_task;
 
-  if (gtd_task_row_get_new_task_mode (row1))
+  if (row1 == self->priv->new_task_row)
     return 1;
-  else if (gtd_task_row_get_new_task_mode (row2))
+  else if (row2 == self->priv->new_task_row)
     return -1;
 
   row1_task = row2_task = NULL;
 
   if (row1)
-    row1_task = gtd_task_row_get_task (row1);
+    row1_task = gtd_task_row_get_task (GTD_TASK_ROW (row1));
 
   if (row2)
-    row2_task = gtd_task_row_get_task (row2);
+    row2_task = gtd_task_row_get_task (GTD_TASK_ROW (row2));
 
   return self->priv->sort_func (GTK_LIST_BOX_ROW (row1),
                                 row1_task,
@@ -443,19 +467,19 @@ internal_compare_dnd_rows (GtdTaskListView *self,
                            GtkListBoxRow   *row1,
                            GtkListBoxRow   *row2)
 {
-  GtdTaskRow *row_above, *current_row;
+  GtkListBoxRow *row_above, *current_row;
   gboolean reverse;
 
   if (GTD_IS_DND_ROW (row1))
     {
       row_above = gtd_dnd_row_get_row_above (GTD_DND_ROW (row1));
-      current_row = GTD_TASK_ROW (row2);
+      current_row = row2;
       reverse = FALSE;
     }
   else
     {
       row_above = gtd_dnd_row_get_row_above (GTD_DND_ROW (row2));
-      current_row = GTD_TASK_ROW (row1);
+      current_row = row1;
       reverse = TRUE;
     }
 
@@ -473,18 +497,13 @@ internal_sort_func (GtkListBoxRow   *a,
                     GtkListBoxRow   *b,
                     GtdTaskListView *view)
 {
-  GtdTaskRow *row1, *row2;
-
   if (!view->priv->sort_func)
     return 0;
 
   if (GTD_IS_DND_ROW (a) || GTD_IS_DND_ROW (b))
     return internal_compare_dnd_rows (view, a, b);
 
-  row1 = GTD_TASK_ROW (a);
-  row2 = GTD_TASK_ROW (b);
-
-  return internal_compare_task_rows (view, row1, row2);
+  return internal_compare_task_rows (view, a, b);
 }
 
 static void
@@ -760,9 +779,16 @@ gtd_task_list_view__update_done_label (GtdTaskListView *view)
 
   gtk_revealer_set_reveal_child (GTK_REVEALER (view->priv->revealer), view->priv->complete_tasks > 0);
 
-  new_label = g_strdup_printf ("%s (%d)",
-                               _("Done"),
-                               view->priv->complete_tasks);
+  if (view->priv->complete_tasks == 0)
+    {
+      new_label = g_strdup_printf ("%s", _("Done"));
+    }
+  else
+    {
+      new_label = g_strdup_printf ("%s (%d)",
+                                   _("Done"),
+                                   view->priv->complete_tasks);
+    }
 
   gtk_label_set_label (view->priv->done_label, new_label);
 
@@ -812,9 +838,6 @@ task_row_entered_cb (GtdTaskListView *self,
   GtdTaskListViewPrivate *priv = self->priv;
   GtdTask *old_task;
 
-  if (row == priv->new_task_row)
-    return;
-
   old_task = gtd_edit_pane_get_task (priv->edit_pane);
 
   /* Save the task previously edited */
@@ -824,12 +847,19 @@ task_row_entered_cb (GtdTaskListView *self,
       real_save_task (self, old_task);
     }
 
+  set_active_row (self, GTK_WIDGET (row));
+
+  /* If we focused the new task row, only activate it */
+  if (GTD_IS_NEW_TASK_ROW (row))
+    {
+      gtk_revealer_set_reveal_child (priv->edit_revealer, FALSE);
+      return;
+    }
+
   gtd_edit_pane_set_task (priv->edit_pane, gtd_task_row_get_task (row));
 
   gtk_revealer_set_reveal_child (priv->edit_revealer, TRUE);
   gtd_arrow_frame_set_row (priv->arrow_frame, row);
-
-  set_active_row (self, GTK_WIDGET (row));
 }
 
 static void
@@ -887,6 +917,7 @@ insert_task (GtdTaskListView *self,
                           G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
   gtd_task_row_set_list_name_visible (GTD_TASK_ROW (new_row), priv->show_list_name);
+  gtd_task_row_set_due_date_visible (GTD_TASK_ROW (new_row), priv->show_due_date);
 
   g_signal_connect_swapped (new_row,
                             "enter",
@@ -901,6 +932,15 @@ insert_task (GtdTaskListView *self,
   gtk_list_box_insert (priv->listbox,
                        new_row,
                        0);
+
+  /*
+   * Setup a sizegroup to let all the tasklist labels have
+   * the same width.
+   */
+  gtd_task_row_set_sizegroups (GTD_TASK_ROW (new_row),
+                               priv->tasklist_name_sizegroup,
+                               priv->due_date_sizegroup);
+
   gtd_task_row_reveal (GTD_TASK_ROW (new_row));
 }
 
@@ -1010,8 +1050,7 @@ gtd_task_list_view__remove_row_for_task (GtdTaskListView *view,
       if (!GTD_IS_TASK_ROW (l->data))
         continue;
 
-      if (!gtd_task_row_get_new_task_mode (l->data) &&
-          gtd_task_row_get_task (l->data) == task)
+      if (gtd_task_row_get_task (l->data) == task)
         {
           destroy_task_row (view, l->data);
           break;
@@ -1090,10 +1129,7 @@ task_completed_cb (GtdTask         *task,
 
       func = task_complete ? remove_subtasks_of_completed_task : add_subtasks_of_task;
 
-      iterate_subtasks (self,
-                        task,
-                        func,
-                        FALSE);
+      iterate_subtasks (self, task, func, FALSE);
     }
 
   gtk_list_box_invalidate_sort (priv->listbox);
@@ -1122,15 +1158,18 @@ gtd_task_list_view__task_added (GtdTaskList     *list,
 }
 
 static void
-gtd_task_list_view__create_task (GtdTaskRow *row,
-                                 GtdTask    *task,
-                                 gpointer    user_data)
+gtd_task_list_view__create_task (GtdTaskRow  *row,
+                                 GtdTask     *task,
+                                 GtdTaskList *list,
+                                 gpointer     user_data)
 {
   GtdTaskListViewPrivate *priv;
-  GtdTaskList *list;
 
   priv = GTD_TASK_LIST_VIEW (user_data)->priv;
-  list = priv->task_list;
+
+  /* If there's a task list set, always go for it */
+  if (priv->task_list)
+    list = priv->task_list;
 
   /*
    * If there is no current list set, use the default list from the
@@ -1180,6 +1219,10 @@ gtd_task_list_view_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_COLOR:
+      g_value_set_boxed (value, self->priv->color);
+      break;
+
     case PROP_HANDLE_SUBTASKS:
       g_value_set_boolean (value, self->priv->handle_subtasks);
       break;
@@ -1188,6 +1231,10 @@ gtd_task_list_view_get_property (GObject    *object,
       g_value_set_boolean (value, self->priv->show_completed);
       break;
 
+    case PROP_SHOW_DUE_DATE:
+      g_value_set_boolean (value, self->priv->show_due_date);
+      break;
+
     case PROP_SHOW_LIST_NAME:
       g_value_set_boolean (value, self->priv->show_list_name);
       break;
@@ -1211,6 +1258,10 @@ gtd_task_list_view_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_COLOR:
+      gtd_task_list_view_set_color (self, g_value_get_boxed (value));
+      break;
+
     case PROP_HANDLE_SUBTASKS:
       gtd_task_list_view_set_handle_subtasks (self, g_value_get_boolean (value));
       break;
@@ -1219,6 +1270,10 @@ gtd_task_list_view_set_property (GObject      *object,
       gtd_task_list_view_set_show_completed (self, g_value_get_boolean (value));
       break;
 
+    case PROP_SHOW_DUE_DATE:
+      gtd_task_list_view_set_show_due_date (self, g_value_get_boolean (value));
+      break;
+
     case PROP_SHOW_LIST_NAME:
       gtd_task_list_view_set_show_list_name (self, g_value_get_boolean (value));
       break;
@@ -1355,7 +1410,8 @@ listbox_drag_motion (GtkListBox      *listbox,
 {
   GtdTaskListViewPrivate *priv;
   GtkListBoxRow *hovered_row;
-  GtdTaskRow *task_row, *row_above_dnd;
+  GtkListBoxRow *task_row;
+  GtkListBoxRow *row_above_dnd;
   gint row_x, row_y, row_height;
 
   priv = gtd_task_list_view_get_instance_private (self);
@@ -1382,7 +1438,7 @@ listbox_drag_motion (GtkListBox      *listbox,
     goto success;
 
   row_above_dnd = NULL;
-  task_row = GTD_TASK_ROW (hovered_row);
+  task_row = hovered_row;
   row_height = gtk_widget_get_allocated_height (GTK_WIDGET (hovered_row));
   gtk_widget_translate_coordinates (GTK_WIDGET (listbox),
                                     GTK_WIDGET (hovered_row),
@@ -1396,7 +1452,7 @@ listbox_drag_motion (GtkListBox      *listbox,
    * the previous row. Also, when hovering the new task row, only show
    * the dnd row over it (never below).
    */
-  if (row_y < row_height / 2 || gtd_task_row_get_new_task_mode (task_row))
+  if (row_y < row_height / 2 || GTD_IS_NEW_TASK_ROW (task_row))
     {
       gint row_index, i;
 
@@ -1410,15 +1466,12 @@ listbox_drag_motion (GtkListBox      *listbox,
           aux = gtk_list_box_get_row_at_index (GTK_LIST_BOX (priv->listbox), i);
 
           /* Skip DnD, New task and hidden rows */
-          if (GTD_IS_DND_ROW (aux) ||
-              gtd_task_row_get_new_task_mode (GTD_TASK_ROW (aux)) ||
-              (aux && !gtk_widget_get_visible (GTK_WIDGET (aux))))
+          if (!GTD_IS_TASK_ROW (aux) || (aux && !gtk_widget_get_visible (GTK_WIDGET (aux))))
             {
               continue;
             }
 
-
-          row_above_dnd = GTD_TASK_ROW (aux);
+          row_above_dnd = aux;
 
           break;
         }
@@ -1437,7 +1490,7 @@ listbox_drag_motion (GtkListBox      *listbox,
       dnd_widget = gtk_drag_get_source_widget (context);
       dnd_row = gtk_widget_get_ancestor (dnd_widget, GTK_TYPE_LIST_BOX_ROW);
       dnd_task = gtd_task_row_get_task (GTD_TASK_ROW (dnd_row));
-      row_above_task = gtd_task_row_get_task (row_above_dnd);
+      row_above_task = gtd_task_row_get_task (GTD_TASK_ROW (row_above_dnd));
 
       /* Forbid DnD'ing a row into a subtask */
       if (row_above_task && gtd_task_is_subtask (dnd_task, row_above_task))
@@ -1604,10 +1657,25 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
                               FALSE,
                               G_PARAM_READWRITE));
 
+  /**
+   * GtdTaskListView::show-due-date:
+   *
+   * Whether due dates of the tasks are shown.
+   */
+  g_object_class_install_property (
+        object_class,
+        PROP_SHOW_DUE_DATE,
+        g_param_spec_boolean ("show-due-date",
+                              "Whether due dates are shown",
+                              "Whether due dates of the tasks are visible or not",
+                              TRUE,
+                              G_PARAM_READWRITE));
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/todo/ui/list-view.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, arrow_frame);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, dnd_row);
+  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, due_date_sizegroup);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, edit_pane);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, edit_revealer);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, empty_box);
@@ -1616,6 +1684,7 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, done_image);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, done_label);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, new_task_row);
+  gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, tasklist_name_sizegroup);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, viewport);
   gtk_widget_class_bind_template_child_private (widget_class, GtdTaskListView, scrolled_window);
 
@@ -1627,6 +1696,7 @@ gtd_task_list_view_class_init (GtdTaskListViewClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_leave);
   gtk_widget_class_bind_template_callback (widget_class, listbox_drag_motion);
   gtk_widget_class_bind_template_callback (widget_class, listbox_row_activated);
+  gtk_widget_class_bind_template_callback (widget_class, task_row_entered_cb);
   gtk_widget_class_bind_template_callback (widget_class, task_row_exited_cb);
 
   gtk_widget_class_set_css_name (widget_class, "task-list-view");
@@ -1638,10 +1708,11 @@ gtd_task_list_view_init (GtdTaskListView *self)
   self->priv = gtd_task_list_view_get_instance_private (self);
   self->priv->can_toggle = TRUE;
   self->priv->handle_subtasks = TRUE;
+  self->priv->show_due_date = TRUE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  self->priv->active_row = GTK_WIDGET (self->priv->new_task_row);
+  set_active_row (self, GTK_WIDGET (self->priv->new_task_row));
 
   gtk_drag_dest_set (GTK_WIDGET (self->priv->listbox),
                      0,
@@ -1816,6 +1887,8 @@ gtd_task_list_view_set_task_list (GtdTaskListView *view,
   if (priv->task_list == list)
     return;
 
+  gtd_new_task_row_set_show_list_selector (GTD_NEW_TASK_ROW (priv->new_task_row), list == NULL);
+
   /*
    * Disconnect the old GtdTaskList signals.
    */
@@ -1938,6 +2011,62 @@ gtd_task_list_view_set_show_list_name (GtdTaskListView *view,
     }
 }
 
+/**
+ * gtd_task_list_view_get_show_due_date:
+ * @self: a #GtdTaskListView
+ *
+ * Retrieves whether the @self is showing the due dates of the tasks
+ * or not.
+ *
+ * Returns: %TRUE if due dates are visible, %FALSE otherwise.
+ */
+gboolean
+gtd_task_list_view_get_show_due_date (GtdTaskListView *self)
+{
+  g_return_val_if_fail (GTD_IS_TASK_LIST_VIEW (self), FALSE);
+
+  return self->priv->show_due_date;
+}
+
+/**
+ * gtd_task_list_view_set_show_due_date:
+ * @self: a #GtdTaskListView
+ * @show_due_date: %TRUE to show due dates, %FALSE otherwise
+ *
+ * Sets whether @self shows the due dates of the tasks or not.
+ */
+void
+gtd_task_list_view_set_show_due_date (GtdTaskListView *self,
+                                      gboolean         show_due_date)
+{
+  GtdTaskListViewPrivate *priv;
+  GList *children;
+  GList *l;
+
+  g_return_if_fail (GTD_IS_TASK_LIST_VIEW (self));
+
+  priv = gtd_task_list_view_get_instance_private (self);
+
+  if (priv->show_due_date == show_due_date)
+    return;
+
+  priv->show_due_date = show_due_date;
+
+  children = gtk_container_get_children (GTK_CONTAINER (priv->listbox));
+
+  for (l = children; l != NULL; l = l->next)
+    {
+      if (!GTD_IS_TASK_ROW (l->data))
+        continue;
+
+      gtd_task_row_set_due_date_visible (l->data, show_due_date);
+    }
+
+  g_list_free (children);
+
+  g_object_notify (G_OBJECT (self), "show-due-date");
+}
+
 /**
  * gtd_task_list_view_get_show_completed:
  * @view: a #GtdTaskListView
@@ -2015,9 +2144,6 @@ gtd_task_list_view_set_show_completed (GtdTaskListView *view,
               if (!GTD_IS_TASK_ROW (l->data))
                 continue;
 
-              if (gtd_task_row_get_new_task_mode (l->data))
-                continue;
-
               task = gtd_task_row_get_task (l->data);
 
               /* Remove completed tasks, and also tasks with a completed parent */
