@@ -138,10 +138,11 @@ gtd_list_selector_grid_item__render_thumbnail (GtdListSelectorGridItem *item)
    * the user will see when selecting the list.
    */
   tasks = g_list_sort (tasks, (GCompareFunc) gtd_task_compare);
+  width -= padding.left + margin.left + padding.right + margin.right;
 
   pango_layout_set_font_description (layout, font_desc);
   pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
-  pango_layout_set_width (layout, (width - margin.left - margin.right) * PANGO_SCALE * scale_factor);
+  pango_layout_set_width (layout, width * PANGO_SCALE);
 
   /*
    * If the list exists and it's first element is a completed task,
@@ -195,7 +196,7 @@ gtd_list_selector_grid_item__render_thumbnail (GtdListSelectorGridItem *item)
            * If we reach the last visible row, it should draw a
            * "…" mark and stop drawing anything else
            */
-          if (y + font_height + 4 + margin.bottom + padding.bottom > THUMBNAIL_SIZE)
+          if (y + font_height + 4 + margin.bottom + padding.bottom > THUMBNAIL_SIZE * scale_factor)
             {
               pango_layout_set_text (layout,
                                      "…",
@@ -367,10 +368,21 @@ static void
 gtd_list_selector_grid_item_state_flags_changed (GtkWidget     *item,
                                          GtkStateFlags  flags)
 {
+  GtdListSelectorGridItem *self;
+
+  self = GTD_LIST_SELECTOR_GRID_ITEM (item);
+
   if (GTK_WIDGET_CLASS (gtd_list_selector_grid_item_parent_class)->state_flags_changed)
     GTK_WIDGET_CLASS (gtd_list_selector_grid_item_parent_class)->state_flags_changed (item, flags);
 
-  gtd_list_selector_grid_item__update_thumbnail (GTD_LIST_SELECTOR_GRID_ITEM (item));
+  /*
+   * The list might be NULL when the provider has been removed, in which
+   * case :dispose() will be called before this function which will remove
+   * the list reference from GtdListSelectorGridItem.
+   */
+
+  if (self->list)
+    gtd_list_selector_grid_item__update_thumbnail (GTD_LIST_SELECTOR_GRID_ITEM (item));
 }
 
 static GtdTaskList*
@@ -427,6 +439,29 @@ gtd_list_selector_grid_item_finalize (GObject *object)
   G_OBJECT_CLASS (gtd_list_selector_grid_item_parent_class)->finalize (object);
 }
 
+static void
+gtd_list_selector_grid_item_dispose (GObject *object)
+{
+  GtdListSelectorGridItem *self = GTD_LIST_SELECTOR_GRID_ITEM (object);
+
+  if (self->list)
+    {
+      g_signal_handlers_disconnect_by_func (self->list,
+                                            gtd_list_selector_grid_item__notify_ready,
+                                            self);
+      g_signal_handlers_disconnect_by_func (self->list,
+                                            color_changed,
+                                            self);
+      g_signal_handlers_disconnect_by_func (self->list,
+                                            gtd_list_selector_grid_item__task_changed,
+                                            self);
+      g_clear_object (&self->list);
+    }
+
+
+  G_OBJECT_CLASS (gtd_list_selector_grid_item_parent_class)->dispose (object);
+}
+
 static void
 gtd_list_selector_grid_item_get_property (GObject    *object,
                                  guint       prop_id,
@@ -537,6 +572,7 @@ gtd_list_selector_grid_item_class_init (GtdListSelectorGridItemClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->finalize = gtd_list_selector_grid_item_finalize;
+  object_class->dispose = gtd_list_selector_grid_item_dispose;
   object_class->get_property = gtd_list_selector_grid_item_get_property;
   object_class->set_property = gtd_list_selector_grid_item_set_property;
 
